import { Component, OnInit, ViewChild, TemplateRef, ViewContainerRef } from '@angular/core';
import { LANG } from '../../../translate.component';
import { HttpClient } from '@angular/common/http';
import { NotificationService } from '../../../notification.service';
import { HeaderService } from '../../../../service/header.service';
import { AppService } from '../../../../service/app.service';
import { MatDialog } from '@angular/material/dialog';
import { FunctionsService } from '../../../../service/functions.service';
import { of } from 'rxjs/internal/observable/of';
import { catchError, tap, map, exhaustMap, filter, finalize } from 'rxjs/operators';
import { SortPipe } from '../../../../plugins/sorting.pipe';
import { FormControl } from '@angular/forms';
import { ManageDuplicateComponent } from './manage-duplicate/manage-duplicate.component';

@Component({
    selector: 'app-contact-duplicate',
    templateUrl: './contact-duplicate.component.html',
    styleUrls: ['./contact-duplicate.component.scss'],
    providers: [AppService, SortPipe]
})
export class ContactDuplicateComponent implements OnInit {

    @ViewChild('adminMenuTemplate', { static: true }) adminMenuTemplate: TemplateRef<any>;

    lang: any = LANG;
    loading: boolean = true;

    subMenus: any[] = [
        {
            icon: 'fas fa-magic',
            route: '/administration/contacts/duplicates',
            label: this.lang.duplicatesContactsAdmin,
            current: true
        },
        {
            icon: 'fa fa-book',
            route: '/administration/contacts/list',
            label: this.lang.contactsList,
            current: false
        },
        {
            icon: 'fa fa-code',
            route: '/administration/contacts/contactsCustomFields',
            label: this.lang.customFieldsAdmin,
            current: false
        },
        {
            icon: 'fa fa-cog',
            route: '/administration/contacts/contacts-parameters',
            label: this.lang.contactsParameters,
            current: false
        },
        {
            icon: 'fa fa-users',
            route: '/administration/contacts/contacts-groups',
            label: this.lang.contactsGroups,
            current: false
        },
    ];

    contactFields: any = [];

    addCriteriaSelect = new FormControl();

    currentFieldsSearch: any = [];

    currentDuplicateId: string = null;

    duplicatesContacts: any[] = [];

    duplicatesContactsCount: number = 0;
    duplicatesContactsRealCount: number = 0;

    displayedColumns = ['lastname', 'company', 'address'];
    dataSource: any;
    isLoadingResults: boolean = false;
    openedSearchTool: boolean = true;

    constructor(
        public http: HttpClient,
        private notify: NotificationService,
        private headerService: HeaderService,
        public appService: AppService,
        public dialog: MatDialog,
        public functions: FunctionsService,
        private sortPipe: SortPipe,
        private viewContainerRef: ViewContainerRef
    ) { }

    async ngOnInit(): Promise<void> {
        this.headerService.injectInSideBarLeft(this.adminMenuTemplate, this.viewContainerRef, 'adminMenu');
        this.headerService.setHeader(this.lang.contactsDuplicates, '', '');
        await this.getContactFields();
        this.setDefaultSearchCriteria();
        this.loading = false;
    }


    getContactFields() {
        return new Promise((resolve, reject) => {
            this.http.get('../rest/contactsParameters').pipe(
                map((data: any) => {
                    const regex = /contactCustomField_[.]*/g;
                    data.contactsParameters = data.contactsParameters.filter((field: any) => field.identifier.match(regex) === null).map((field: any) => {
                        return {
                            ...field,
                            label: this.lang['contactsParameters_' + field.identifier]
                        };
                    });
                    return data.contactsParameters;
                }),
                tap((fields: any) => {
                    this.contactFields = fields;
                }),
                exhaustMap(() => this.http.get('../rest/contactsCustomFields')),
                map((data: any) => {
                    data.customFields = data.customFields.map((field: any) => {
                        return {
                            ...field,
                            id: `contactCustomField_${field.id}`,
                            identifier: `contactCustomField_${field.id}`
                        };
                    });
                    return data.customFields;
                }),
                tap((fields: any) => {
                    this.contactFields = this.contactFields.concat(fields);
                    this.contactFields = this.sortPipe.transform(this.contactFields, 'label');
                    resolve(true);
                }),
                catchError((err: any) => {
                    this.notify.handleSoftErrors(err);
                    return of(false);
                })
            ).subscribe();
        });
    }

    setDefaultSearchCriteria(target: string[] = ['lastname', 'company']) {
        this.contactFields.filter((contact: any) => target.indexOf(contact.identifier) > -1).forEach((field: any) => {
            field.disabled = true;
            this.currentFieldsSearch.push(field);
        });
    }

    addCriteria(id: any) {
        this.contactFields.filter((contact: any) => contact.id === id).forEach((field: any) => {
            field.disabled = true;
            this.currentFieldsSearch.push(field);
        });
        this.addCriteriaSelect.reset();
    }

    removeCriteria(field: any) {
        this.contactFields.forEach((contact: any, index: number) => {
            if (contact.id === field.id) {
                this.currentFieldsSearch = this.currentFieldsSearch.filter((currField: any) => currField.id !== field.id);
                contact.disabled = false;
            }
        });
    }

    searchDuplicates() {
        this.duplicatesContacts = [];
        this.isLoadingResults = true;
        const queryParam = '?criteria[]=' + this.currentFieldsSearch.map((field: any) => field.identifier).join('&criteria[]=');
        this.http.get(`../rest/duplicatedContacts${queryParam}`).pipe(
            map((data: any) => {
                this.duplicatesContactsRealCount = data.realCount;
                this.duplicatesContactsCount = data.returnedCount;
                data.contacts.forEach((element: any, index: number) => {
                    if (index === 0) {
                        element.odd = true;
                    } else if (data.contacts[index - 1] !== undefined && data.contacts[index - 1].duplicateId === element.duplicateId) {
                        element.odd = data.contacts[index - 1].odd;
                    } else {
                        element.odd = !data.contacts[index - 1].odd;
                    }
                    const tmpFormatedAddress = [];
                    tmpFormatedAddress.push(element.addressNumber);
                    tmpFormatedAddress.push(element.addressStreet);
                    tmpFormatedAddress.push(element.addressPostcode);
                    tmpFormatedAddress.push(element.addressTown);
                    tmpFormatedAddress.push(element.addressCountry);
                    element.address = tmpFormatedAddress.filter(address => !this.functions.empty(address)).join(' ');
                });
                return data.contacts;
            }),
            tap((contacts: any) => {
                console.log(contacts);

                this.duplicatesContacts = contacts;
                setTimeout(() => {
                    this.displayedColumns = this.currentFieldsSearch.map((field: any) => field.identifier);
                    this.displayedColumns.push('address');
                    this.openedSearchTool = false;
                }, 0);
            }),
            finalize(() => this.isLoadingResults = false),
            catchError((err: any) => {
                this.notify.handleSoftErrors(err);
                return of(false);
            })
        ).subscribe();
    }

    selectDuplicates(duplicateId: string) {
        this.currentDuplicateId = duplicateId;
    }

    manageDuplicate(duplicateId: string) {
        const dialogRef = this.dialog.open(ManageDuplicateComponent, {
            panelClass: 'maarch-modal',
            data: { duplicate: this.duplicatesContacts.filter((contact: any) => contact.duplicateId === duplicateId).map((contact: any) => ({ id: contact.id, type: 'contact'})) }
        });
        dialogRef.afterClosed().pipe(
            filter((data: any) => data === 'success'),
            tap(() => {
                this.notify.success(this.lang.contactsMerged);
                this.duplicatesContactsCount--;
                this.duplicatesContactsRealCount--;
                this.duplicatesContacts = this.duplicatesContacts.filter((contact: any) => contact.duplicateId !== duplicateId);
            }),
            catchError((err: any) => {
                this.notify.handleSoftErrors(err);
                return of(false);
            })
        ).subscribe();
    }
}
